package trains.persistence;

import trains.model.*;

import java.time.DayOfWeek;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.List;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;
import java.util.Map;
import java.util.Optional;
import java.util.StringTokenizer;

public class MyTrainReader implements TrainReader {
	private static DateTimeFormatter timeFormatter = DateTimeFormatter
			.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY);

	private Map<String, Station> stations;

	public MyTrainReader(Map<String, Station> stations) {
		this.stations = stations;
	}

	// associa l'ID di un treno al treno stesso
	public Collection<Train> readTrains(Reader reader) throws IOException,
			BadFileFormatException {
		// Ogni treno  descritto da una riga, strutturata in una serie di campi
		// separati da tabulazioni:
		// - identificativo univoco del Train (il primo carattere indica
		// loperatore, il resto  numerico)
		// - giorni di effettuazione (1=luned,, 7=domenica; i giorni di non
		// effettuazione sono semplicemente omessi)
		// - identificativi univoci delle stazioni servite, ciascuna con i
		// rispettivi orari di arrivo e partenza
		// espressi nella forma hh.mm (fanno eccezione la Station di origine e
		// la Station finale, che hanno un orario solo).
		// Se in una Station non  consentita la salita o la discesa, il
		// corrispondente orario  --.
		// Stazioni e orari sono separati fra loro da barre, mentre i blocchi
		// Station/orari sono separati fra loro da virgole

		BufferedReader bufferedReader = new BufferedReader(reader);
		ArrayList<Train> trains = new ArrayList<Train>();
		String line = null;
		while ((line = bufferedReader.readLine()) != null) {
			Train t = extractTrain(line);
			trains.add(t);
		}
		bufferedReader.close();
		return trains;
	}

	private Train extractTrain(String line) throws BadFileFormatException {
		StringTokenizer st = new StringTokenizer(line, "\t/,\n\r");

		String id = st.nextToken().trim().toUpperCase(); // operatore + numero
		if (id.isEmpty())
			throw new BadFileFormatException("ID treno mancante");
		Operator operator = null;
		try {
			operator = Operator.valueOf(id.substring(0, 1));
		} catch (IllegalArgumentException e) {
			throw new BadFileFormatException("operatore inesistente");
		}

		long number = -1;
		try {
			number = Integer.parseInt(id.substring(1));
		} catch (NumberFormatException e) {
			throw new BadFileFormatException("formato numero treno errato");
		}

		String daysString = st.nextToken().trim().toUpperCase(); // giorni di effettuazione
		if (daysString.isEmpty())
			throw new BadFileFormatException("giorni effettuazione mancanti");
		if (daysString.contains("0"))
			throw new BadFileFormatException("formato giorni effettuazione errato");
		try {
			Integer.parseInt(daysString, 8); // verifico che siano numeri fra 1 e 7 (0  gi stato scartato prima)
		} catch (NumberFormatException e) {
			throw new BadFileFormatException("formato giorni effettuazione errato");
		}
		DayOfWeek[] days = new DayOfWeek[daysString.length()];
		for (int i = 0; i < days.length; i++)
			days[i] = DayOfWeek.of(Integer.parseInt(daysString.substring(i,
					i + 1)));

		List<Stop> stops = extractStops(st);

		return new Train(operator, number, days, stops);
	}

	private List<Stop> extractStops(StringTokenizer st)
			throws BadFileFormatException {
		// BO00/11.23,RE01/11.42/11.44,MI02/12.22/12.24, ... TO01/13.35/--,TO00/13.45
		List<Stop> stops = new ArrayList<>();
		try {
			while (st.hasMoreTokens()) {
				String stationID = st.nextToken().trim();
				String orarioA = st.nextToken().trim();
				String orarioP = st.nextToken().trim();
				Optional<LocalTime> timeA = orarioA.equals("--") ? Optional.empty() : Optional.of(LocalTime.parse(orarioA, timeFormatter));
				Optional<LocalTime> timeP = orarioP.equals("--") ? Optional.empty() : Optional.of(LocalTime.parse(orarioP, timeFormatter));
				Station currentStation = stations.get(stationID);
				if (currentStation == null)
					throw new BadFileFormatException("ID stazione sconosciuto");
				stops.add(new Stop(currentStation, timeA, timeP));
			}
		} catch (Exception e) {
			throw new BadFileFormatException(e);
		}
		return stops;
	}
}